import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import math
from sklearn import metrics


from sklearn.preprocessing import MinMaxScaler
from sklearn.metrics import mean_squared_error

from keras.models import Sequential
from keras.layers import Dense
from keras.layers import SimpleRNN

plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['font.family']='sans-serif'
plt.rcParams['axes.unicode_minus'] = False

def create_dataset(dataset, look_back=1):
    dataX, dataY = [], []
    for i in range(len(dataset)-look_back-1):
        a = dataset[i:(i+look_back), 0]
        dataX.append(a)
        dataY.append(dataset[i+look_back, 0])
    return np.array(dataX), np.array(dataY)

def mape(y_true, y_pred):
    return np.mean(np.abs((y_pred - y_true) / y_true))

def smape(y_true, y_pred):
    return 2.0 * np.mean(np.abs(y_pred - y_true) / (np.abs(y_pred) + np.abs(y_true))) * 100

data = pd.read_csv("Moore_values.txt")
data1 = pd.read_csv("best_data.txt")
print(data)
n = []
for i in range(len(data)):
    n.append(i+1)
print(n)
data['a']=n
print(data)
dataset = data['b'].values.reshape(-1,1)
dataset = dataset.astype('float32')
print(dataset)

train_size = int(len(dataset) * 0.8)
test_size = len(dataset) - train_size
train, test = dataset[0:train_size, :], dataset[train_size:len(dataset), :]
print(len(train), len(test))

look_back = 1
trainX, trainY = create_dataset(train, look_back=look_back)
testX, testY = create_dataset(test, look_back=look_back)

print(trainX)

trainX = np.reshape(trainX, (trainX.shape[0], 1,trainX.shape[1]))
testX =np.reshape(testX, (testX.shape[0], 1, testX.shape[1]))

model = Sequential()
#model.add(SimpleRNN(32, input_shape=(1, look_back)))
model.add(SimpleRNN(units=32, return_sequences=True, input_shape=(1, look_back)))
model.add(SimpleRNN(units=16, return_sequences=True))
#model.add(SimpleRNN(units=16, return_sequences=True))

model.add(SimpleRNN(units=16))
model.add(Dense(1))
model.compile(loss='mean_squared_error', optimizer='adam')
hist = model.fit(trainX, trainY, epochs=100, batch_size=32, verbose=2)

epochs = 100
training_loss = hist.history['loss']
xc = range(epochs)
# plt.figure(1, figsize=(5, 5))
plt.plot(xc, training_loss)
plt.show()


trainPredict = model.predict(trainX)
# print(trainPredict)
# print("------------------------------------")
# print("------------------------------------")
testPredict = model.predict(testX)
print(testPredict)

"""
trainPredict = scaler.inverse_transfom(trainPredict)
trainY = scaler.inverse_transfom([trainY])
testPredict = scaler.inverse_transfom(testPredict)
testY = scaler.inverse_transfom([testY])
"""

trainScore = math.sqrt(mean_squared_error(trainY, trainPredict))
print('Train Score: %.6f RMSE' % (trainScore))
testScore = math.sqrt(mean_squared_error(testY, testPredict))
print('Test Score: %.6f RMSE' % (testScore))
testScore2 = metrics.mean_squared_error(testY, testPredict)
print('Test Score: %.6f MSE' % (testScore2))
testScore3 = metrics.median_absolute_error(testY, testPredict)
print('Test Score: %.6f MAE' % (testScore3))
testScore4 = mape(testY, testPredict)
print('Test Score: %.6f MAPE' % (testScore4))
testScore5 = smape(testY, testPredict)
print('Test Score: %.6f SMAPE' % (testScore4))
print("-----------------------------------------")
print("-----------------------------------------")
"""
testScore = math.sqrt(mean_squared_error(testY, data1))
print('Test Score: %.6f RMSE' % (testScore))
testScore2 = metrics.mean_squared_error(testY, data1)
print('Test Score: %.6f MSE' % (testScore2))
testScore3 = metrics.median_absolute_error(testY, data1)
print('Test Score: %.6f MAE' % (testScore3))
testScore4 = mape(testY, data1)
print('Test Score: %.6f MAPE' % (testScore4))
testScore5 = smape(testY, data1)
print('Test Score: %.6f SMAPE' % (testScore4))
"""
# 读取指定列
m = []
for i in range(len(testX)):
    m.append(i+1)
xdata = []
ydata = []
xdata = m
ydata_1 = testY
ydata_2 = testPredict

# 设置格式
plt.plot(xdata, ydata_1, 'bo-', label=u'', linewidth=1)
plt.plot(xdata, ydata_2, 'ro-', label=u'', linewidth=1)
#plt.title(u"", size=10)   #设置表名为“表名”
plt.legend()
plt.xlabel(u'时间/天', size=10)   #设置x轴名为“x轴名”
plt.ylabel(u'流量态势值', size=10)   #设置y轴名为“y轴名”
plt.show()